import os
import unittest
from unittest.mock import Mock, patch

from generalindex.DatalakeHandler import DatalakeHandler

os.environ["NG_API_AUTHTOKEN"] = "test"


class DatalakeHandlerTestCase(unittest.TestCase):
    def test_given_avoid_duplicates_when_upload_file_returns_with_fields(self):
        dh = DatalakeHandler()
        mock_caller = Mock()
        test_id = "test_id"
        mock_caller.file_uploader.side_effect = [test_id]
        with patch.object(dh, "caller", new=mock_caller):
            result = dh.upload_file("Test,File,Content", "test_group", "test.csv", 'SOURCE', avoid_duplicates=True)
            calls = mock_caller.mock_calls
            self.assertTrue(result[0] is not None)
            self.assertTrue(result[0] == test_id)
            self.assertTrue(calls[0].args[0]['fields'][0]['name'] == "AVOID_DUPLICATES")
            self.assertTrue(calls[0].args[0]['fields'][0]['value'] == 'true')
    def test_given_partial_update_when_upload_file_returns_with_fields(self):
        dh = DatalakeHandler()
        mock_caller = Mock()
        test_id = "test_id"
        mock_caller.file_uploader.side_effect = [test_id]
        with patch.object(dh, "caller", new=mock_caller):
            result = dh.upload_file("Test,File,Content", "test_group", "test.csv", 'SOURCE', partial_update=True)
            calls = mock_caller.mock_calls
            self.assertTrue(result[0] is not None)
            self.assertTrue(result[0] == test_id)
            self.assertTrue(calls[0].args[0]['fields'][0]['name'] == "PARTIAL_UPDATE")
            self.assertTrue(calls[0].args[0]['fields'][0]['value'] == 'true')



from . import LOGGER_NAME
from .HTTPCaller import HTTPCaller
from deprecated import deprecated
import time
import logging
import os
log = logging.getLogger(LOGGER_NAME)

class StatusHandler:
    '''
    Allows to send statuses to the Status Page of the NorthGravity Application.
    Each status type has a color and a corresponding message.
    - INFO: Blue
    - ERROR: Red
    - WARNING: Orange
    '''

    def __init__(self):
        self.caller = HTTPCaller()

        self.job_id = os.environ.get('JOBID')
        self.eid = os.environ.get('EID')
        self.component_name = os.environ.get('NG_COMPONENT_NAME')
        self.group_name = os.environ.get('NG_STATUS_GROUP_NAME', '')

    def send_status(self, status, message):
        # Set the api call for the status connected to the new log event
        log.debug(f'Send status with status {status} and message {message}')

        payload = {'jobId': self.job_id,
                   'eId': self.eid,
                   'status': status,
                   'message': message,
                   'className': 'python',
                   'groupName': self.group_name,
                   'componentName': self.component_name,
                   'time': time.strftime('%Y-%m-%dT%H:%M:%S'),
                   }

        r = self.caller.post(path="/status", payload=payload, headers={'X-KH-E-ID': self.eid})

        log.debug(f'URL: {r.url} \nHeaders: {r.request.headers} \nPayload: {payload}')

        if r.status_code > 201:
            log.info("Sending status response was: " + str(r.status_code) + " and text " + str(r.text))

        return

    def running(self, message):
        self.send_status('RUNNING', message)

    @deprecated(version='0.1.5', reason="You should use running(message) or error(message) function instead.")
    def info(self, message):
        self.send_status('RUNNING', message)

    def error(self, message):
        self.send_status('ERROR', message)

    def warn(self, message):
        self.send_status('WARN', message)



import base64
from . import *
import os
import logging

log = logging.getLogger(LOGGER_NAME)


class Authenticator:
    def __init__(self):       
        self.token = os.getenv('NG_API_AUTHTOKEN')
        self.login = os.getenv('LOGIN')
        self.password = os.getenv('PASSWORD')

    def get_token(self):
        if self.token is not None:
            if self.token.startswith("Bearer") or self.token.startswith("Basic") or self.token.startswith("ApiKey"):
                log.debug('Authentication with Web Token ...')
                return {'Authorization': f'{self.token}'}
            else:
                log.debug('Authentication with Web Token ...')
                return {'Authorization': f'Bearer {self.token}'}

        else:
            if self.login is not None and self.password is not None:
                log.debug('Authentication with Credentials ...')
                credentials = self.login + ":" + self.password
                message_bytes = credentials.encode('ascii')
                base64_enc = base64.b64encode(message_bytes).decode('UTF-8')

                d = {'Authorization': f"Basic {base64_enc}"}
                return d

            else:
                log.error('No login or password provided, neither token can be read from environment variable.')
                raise Exception('Authentication issue : no credentials found')



import os

import setuptools


class CleanCommand(setuptools.Command):
    """Custom clean command to tidy up the project root."""
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system('rm -vrf ./build ./dist ./*.pyc ./*.tgz ./*.egg-info')


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="generalindex",
    version="0.1.15",
    author="General Index",
    author_email="info@general-index.com",
    description="Python SDK for the General Index platform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.general-index.com/",
    packages=setuptools.find_packages(),
    classifiers=["Programming Language :: Python :: 3",
                 "License :: OSI Approved :: MIT License",
                 "Operating System :: OS Independent"
                 ],
    install_requires=[
        'requests>=2.31.0',
        'urllib3>=2.1.0',
        'deprecated>=1.2.14'
    ],
    python_requires='>=3.6',
    cmdclass={
        'clean': CleanCommand,
    }
)
